# Include the BeakerLib environment
. /usr/share/beakerlib/beakerlib.sh

# Set the full test name
TEST="testD"

# Package being tested
PACKAGE="autofs"

# Required packages.
PACKAGES=( "openldap"         \
           "openldap-clients" \
           "openldap-servers" \
  	   "autofs"           \
           "rsyslog"          )

rlJournalStart

  rlPhaseStartSetup

    # Backup service. 
    rlServiceStop "autofs"
    rlServiceStop "slapd"
    rlServiceStop "rsyslog"

    # Check & install reqiured packages.
    yum clean all
    for P in ${PACKAGES[@]}; do
        rlCheckRpm $P || rlRun "yum -y install $P" 0 "Installing $P"
    done

    # Backup configuration and database.
    rlFileBackup "$HOME/ldaprc" && rlRun "rm -f $HOME/ldaprc" 0
    rlFileBackup "/etc/hosts"
    rlFileBackup "/etc/rsyslog.conf"
    rlFileBackup "/etc/openldap/slapd.conf"
    rlFileBackup "/etc/openldap/ldap.conf"
    rlFileBackup "/etc/sysconfig/ldap"
    rlFileBackup "/etc/openldap/cacerts/" && rlRun "rm -rf /etc/openldap/cacerts/*" 0
    rlFileBackup "/etc/openldap/slapd.d/" && rlRun "rm -rf /etc/openldap/slapd.d/*" 0
    rlFileBackup "/var/run/openldap/" && rlRun "rm -rf /var/run/openldap/*" 0
    rlFileBackup "/var/lib/ldap/" && rlRun "rm -rf /var/lib/ldap/*" 0
    rlFileBackup "/etc/nsswitch.conf"
    rlFileBackup "/etc/openldap/schema"

    # Back-up autofs configuration
    rlFileBackup "/etc/sysconfig/autofs" && rlRun "rm -f /etc/sysconfig/autofs" 0
    rlFileBackup "/etc/autofs_ldap_auth.conf" && rlRun "rm -f /etc/autofs_ldap_auth.conf" 0
    
    # Enable logging.
    rlRun "echo 'local4.* /var/log/slapd.log' >> /etc/rsyslog.conf" 0
    rlRun "service rsyslog start && sleep 5"

    # Configure.
    rlRun "echo '127.0.0.1 my-domain.com' >> /etc/hosts" 0
    rlRun "cp -f ldaprc $HOME/ldaprc" 0
    rlRun "cp -f autofs.schema /etc/openldap/schema/autofs.schema" 0
    rlRun "cp -f nsswitch.conf /etc/nsswitch.conf" 0
    rlRun "cp -f ldap.conf /etc/openldap/ldap.conf" 0
    rlRun "cp -f slapd.conf /etc/openldap/slapd.conf" 0
    rlRun "cp -f *.pem /etc/openldap/cacerts/" 0
    rlRun "chown ldap:ldap /etc/openldap/cacerts/*" 0
    rlRun "chmod 400 /etc/openldap/cacerts/*" 0
    rlRun "slaptest -f /etc/openldap/slapd.conf -F /etc/openldap/slapd.d/" 0-255
    rlRun "chown -R ldap:ldap /etc/openldap/slapd.d/* /var/lib/ldap/*" 0
    rlRun "cacertdir_rehash /etc/openldap/cacerts/" 0

    # Setup loop device
    rlRun "dd if=/dev/zero of=/tmp/test count=10000 bs=1024" 0 "Create 10MB /tmp/test file"
    rlRun "losetup /dev/loop0 /tmp/test" 0 "Connect with loop device"
    rlRun "mkfs.ext4 /dev/loop0" 0 "Create file system"

    # SELinux cleaning.
    rlRun "semanage fcontext -f -l -a -t autofs_t /usr/tmp" 0
    rlRun "restorecon -vvRF /etc/ /usr/"
    
  rlPhaseEnd

  rlPhaseStartTest  

    # Testing.

    # No TLS/SSL
    rlRun "cp -f autofs-no_tls_ssl /etc/sysconfig/autofs" 0
    rlRun "cp -f ldap-tls_or_no_tls /etc/sysconfig/ldap" 0
    rlRun "cp -f autofs_ldap_auth-ldap-no_tls_ssl.conf /etc/autofs_ldap_auth.conf" 0
    rlRun "chmod 0600 /etc/autofs_ldap_auth.conf" 0
    rlRun "service slapd start && sleep 5" 0
    rlRun "ldapadd -D cn=Manager,dc=my-domain,dc=com -w x -x -H ldap://my-domain.com -f data.ldif" 0
    rlRun "service autofs start && sleep 5" 0
    rlRun "ls /misc/loop" 0
    rlRun "service autofs stop && sleep 5" 0

    # TLS
    rlRun "cp -f autofs-tls /etc/sysconfig/autofs" 0
    rlRun "cp -f autofs_ldap_auth-tls.conf /etc/autofs_ldap_auth.conf" 0
    rlRun "chmod 0600 /etc/autofs_ldap_auth.conf" 0
    rlRun "service autofs start && sleep 5" 0
    rlRun "ls /misc/loop" 0
    rlRun "ldapsearch -H ldap://my-domain.com -x -ZZ '*'" 0 "TLS"
    rlRun "service autofs stop && sleep 5" 0
    rlRun "service slapd stop && sleep 5" 0
  
    # SSL
    rlRun "cp -f autofs-ssl /etc/sysconfig/autofs" 0
    rlRun "cp -f ldap-ssl /etc/sysconfig/ldap" 0
    rlRun "cp -f autofs_ldap_auth-ssl.conf /etc/autofs_ldap_auth.conf" 0
    rlRun "chmod 0600 /etc/autofs_ldap_auth.conf" 0
    rlRun "service slapd start && sleep 5" 0
    rlRun "ldapmodify -D cn=Manager,dc=my-domain,dc=com -w x -x -H ldaps://my-domain.com -f data-ssl.ldif" 0
    rlRun "service autofs start && sleep 5" 0
    rlRun "ls /misc/loop" 0
    rlRun "service autofs stop && sleep 5" 0
    rlRun "service slapd stop && sleep 5" 0

  rlPhaseEnd

  rlPhaseStartCleanup
  
    # Clean up loop device setup
    rlRun "losetup -d /dev/loop0" 0 "Unconnect from loop device"
    rlRun "rm /tmp/test" 0 "Remove /tmp/test file"

    rlRun "rm -f $HOME/ldaprc" 0
    
    # Restore configuration backup.
    rlFileRestore

    rlRun "semanage fcontext -f -l -d -t autofs_t /usr/tmp" 0
    rlRun "restorecon -vvRF /etc/ /usr/"
   
    # Restore service backup.
    rlServiceRestore "slapd"
    rlServiceRestore "rsyslog"
    rlServiceRestore "autofs"

  rlPhaseEnd

rlJournalEnd

# Print the test report
rlJournalPrintText
